@@ -7,7 +7,7 @@ from prompt_toolkit.enums import DEFAULT_BUFFER, SEARCH_BUFFER
 from prompt_toolkit.filters import IsDone, HasCompletions, RendererHeightIsKnown, HasFocus, Condition
 from prompt_toolkit.key_binding.vi_state import InputMode
 from prompt_toolkit.layout.containers import Window, HSplit, VSplit, FloatContainer, Float, ConditionalContainer, ScrollOffsets
-from prompt_toolkit.layout.controls import BufferControl, TokenListControl, FillControl
+from prompt_toolkit.layout.controls import BufferControl, TokenListControl, FillControl, UIContent
 from prompt_toolkit.layout.dimension import LayoutDimension
 from prompt_toolkit.layout.lexers import SimpleLexer
 from prompt_toolkit.layout.margins import PromptMargin
@@ -19,7 +19,7 @@ from prompt_toolkit.layout.utils import token_list_width
 from prompt_toolkit.reactive import Integer
 from prompt_toolkit.selection import SelectionType
 
-from .filters import HasSignature, ShowSidebar, ShowSignature, ShowDocstring,ShowRealtimeInput
+from .filters import HasSignature, ShowSidebar, ShowSignature, ShowDocstring,ShowRealtimeInput,ShowVarSpace,ShowVarSpaceAndShowRealtimeInput,ShowVarSpaceOrShowRealtimeInput,ShowParenthesisAutomator
 from .utils import if_mousedown
 
 from pygments.lexers import PythonLexer
@@ -431,19 +431,23 @@ def show_sidebar_button_info(python_input):
     token = Token.Toolbar.Status
 
     version = sys.version_info
-    tokens = [
+    import datetime
+    remove_beginning_zero=lambda x:x[1:] if x[0] == '0' else x
+    get_tokens=lambda cli: [
+        (token,(remove_beginning_zero(datetime.datetime.now().strftime("%I:%M%p")).lower())),# put the time here
+        (token, ' '),
         (token.Key, '[F2]', toggle_sidebar),
         (token, ' Menu', toggle_sidebar),
-        (token, ' - '),
-        (token.PythonVersion, '%s %i.%i.%i' % (platform.python_implementation(),
-                                               version[0], version[1], version[2])),
+        # (token, ' - '),
+        # (token.PythonVersion, '%s %i.%i.%i' % (platform.python_implementation(),
+        #                                        version[0], version[1], version[2])),
         (token, ' '),
     ]
-    width = token_list_width(tokens)
+    width = token_list_width(get_tokens(None))
 
-    def get_tokens(cli):
-        # Python version
-        return tokens
+    # def get_tokens(cli):
+    #     Python version
+        # return tokens
 
     return ConditionalContainer(
         content=Window(
@@ -507,7 +511,7 @@ def create_layout(python_input,
     extra_body = [extra_body] if extra_body else []
     extra_toolbars = extra_toolbars or []
     extra_buffer_processors = extra_buffer_processors or []
-    input_buffer_height = input_buffer_height or D(min=6)
+    input_buffer_height = input_buffer_height or D(min=3)
 
     def create_python_input_window():
         def menu_position(cli):
@@ -557,6 +561,19 @@ def create_layout(python_input,
                         else input_buffer_height)),
             wrap_lines=Condition(lambda cli: python_input.wrap_lines),
         )
+    def title_fill(title,fill=' ',style=Token.Window.TIItleV2):
+        height=title.count("\n")+1
+        import rp.r_iterm_comm
+        rp.r_iterm_comm.python_input_buffers[title]=title
+        try:
+            return VSplit([
+            Window(height=D.exact(height),content=FillControl(fill,token=style)),
+            Window(BufferControl(buffer_name=title,lexer=SimpleLexer(default_token=style),),wrap_lines=False,height=D.exact(height),width=D.exact(max(len(x)for x in title.split('\n')))),
+            Window(height=D.exact(height),content=FillControl(fill,token=style)),
+                    ])
+        except:
+            return Window(height=D.exact(1),content=FillControl(fill,token=Token.Separator))
+    import rp
     return HSplit([
                       VSplit([
                           HSplit([
@@ -608,20 +625,48 @@ def create_layout(python_input,
                                   filter=HasSignature(python_input) & ShowDocstring(python_input) & ~IsDone(),
                               ),
                               # realtime display region RYAN BURGERT CODE zone
+                              # ConditionalContainer(
+                              #     content=Window(height=D.exact(1),content=FillControl('\u2500',token=Token.Separator)),
+                              #     filter=ShowVarSpaceOrShowRealtimeInput(python_input) & ~IsDone()),
                               ConditionalContainer(
-                                  content=Window(height=D.exact(1),
-                                                 content=FillControl('\u2500',token=Token.Separator)),
-                                  filter=ShowRealtimeInput(python_input) & ~IsDone()),
-                              ConditionalContainer(
-                                  content=Window(
-                                      BufferControl(
-                                          buffer_name='realtime_display',
-                                          lexer=SimpleLexer(default_token=Token.Docstring),
-                                          # lexer=PythonLexer,
-                                      ),
-                                      height=D(max=12)),
-                                  filter=ShowRealtimeInput(python_input) & ~IsDone(),
+
+
+                                  content=
+                                  HSplit([
+                                      title_fill("Parenthesis Automator"),
+                                      # Window(BufferControl(buffer_name='parenthesizer_buffer',lexer=lexer,),wrap_lines=False,height=D(max=rp.r_iterm_comm.parenthesized_line.count('\n'),min=rp.r_iterm_comm.parenthesized_line.count('\n')    ))
+                                      Window(BufferControl(buffer_name='parenthesizer_buffer',lexer=lexer,),wrap_lines=False)
+                                      # title_fill(rp.r_iterm_comm.parenthesized_line),
+                                  ]),
+                                  filter=ShowParenthesisAutomator(python_input) & ~IsDone(),
                               ),
+                              VSplit([
+                                  ConditionalContainer(
+                                      content=
+                                      HSplit([
+                                          title_fill("Realtime Evaluator"),
+                                          Window(BufferControl(buffer_name='realtime_display',lexer=SimpleLexer(default_token=Token.Docstring),),wrap_lines=True,height=D(weight=2))
+                                      ]),
+                                      filter=ShowRealtimeInput(python_input) & ~IsDone(),
+                                  ),
+                                  ConditionalContainer(
+                                      content=Window(width=D.exact(1),
+                                                     content=FillControl('│',token=Token.Window.TIItleV2)),
+                                      filter=ShowVarSpaceAndShowRealtimeInput(python_input) & ~IsDone()),
+                                  ConditionalContainer(
+
+
+                                      content=
+                                      HSplit([
+                                          title_fill("VARS"),
+                                          Window(BufferControl(buffer_name='vars',
+                                                               lexer=SimpleLexer(default_token=Token.Docstring)
+                                                               # lexer=lexer
+                                                               ,),wrap_lines=True)
+                                      ]),
+                                      filter=ShowVarSpace(python_input) & ~IsDone(),
+                                  ),
+                                ]),
                           ]),
                           HSplit([
                               python_sidebar(python_input),
