@@ -8,6 +8,111 @@ When it makes sense, - will be turned into _.
 		if -    if _
 
 
+#PLANS:
+	#Pretty much all future plans are here purely to make it clean, and to increase adoptability...
+	#Maybe put this in my apple Notes app later...so as not to clutter the help document lol.
+	#Ultimate terminal IDE plans:
+	#	Probably won't get around to multi-cursors, realistically. Vim can handle that anyway.
+	#	Alt+up that correctly expands and decreases selection region
+	#	In editor: Right click items OR some keyboard shortcut for context menus, that include:
+	#		Viewing documentation for a function/class etc
+	#		Copying it
+	#		Renaming it
+	#		Extracting selection to a variable
+	#		Finding usages (don't know how we would show results yet tho)
+	#	Better syntax highlighting: 
+	#		Highlight callables differently. We can use JEDI for this.
+	#	GUI menus for everything. 
+	#	Multiple tabs for the buffer; where we can open files and save files without having to type them out every time.
+	#	Sync breakpoints with PUDB in the gutter of the editor. Remember that the buffer is refreshed every time, so this might be difficult...
+	#	
+
+
+
+##################CSE526 PLANS
+
+	* Keep valid syntax as much as possible. New way to write def:
+	* Need to somehow have more memory than just the visible text in order for this to work well...
+	* Temporary text created in this way should be highlighted differently (temporary text, that might "solidify" if we move the cursor away)
+
+	*Utilizes concrete syntax trees and differentiable syntax errors
+
+	EXAMPLE (of how things could be done differently)
+
+	On ‹ ›: ‹d¦›   --->   ‹def _¦():
+	                           pass›
+
+	Then, On ‹f›:  ‹def _¦():   --->   ‹def f¦():
+	                    pass›               pass›
+
+	Or,   On ‹_›:  ‹def _¦():   --->   ‹def _¦():   // This is because, although you can't see it in this document, that _ changed colors. It used to be a temp token, but now it's solidified. All state is still visible in the terminal (by color); this might be accomplished by also having state in the text there (with special unicode syntax + a postprocessor so that you don't see it + special syntax highlighting. Maybe an invisible no-space character before another character changes its color?)
+	                    pass›               pass›
+
+	Then, On ‹↵›:  ‹def f¦():   --->   ‹def f():
+	                    pass›               ¦
+	                                        pass›
+
+	Then, On ‹p›:  ‹def f():   --->   ‹def f():
+	                    ¦                  p¦›
+	                    pass›               
+
+	Or,   On ‹↵›:  ‹def f():   --->   ‹def f():
+	                    ¦                  pass
+	                    pass›          ¦›
+
+	ANOTHER EXAMPLE:
+
+	On ‹ ›: ‹if¦›   --->   ‹if ¦:
+
+	#Play around with this to see how we can separate the syntax-error region from the rest of the code
+	#For example, while entering some python code keep pressing ^e over and over to see how the syntax errors evolve over time
+	#	For example, enter [x for x in y|] then press ^e then type ' if' to get [x for x in y if|] then press ^e again to see how ' if' is reddened
+	#This will let us use a concrete syntax tree to analyze python even when there are syntax errors.
+	#	For example, this will let alt+up progressively select text more easily.
+	#TODO: Pull request changes to PromptToolkit: Mouse events, faster regions, color filters - to have highlightable menus when moused over
+	def threeway_split(old_text,new_text):
+		#EXAMPLES:
+		#     >>> threeway_split('Hello World','Hello Fair World')
+		#    ans = ('Hello ', 'Fair ', 'World')
+		#     >>> threeway_split('Hello World','HelloWorld')
+		#    ans = ('Hello', '', 'World')
+		#     >>> threeway_split('Hello World','HelWorld')
+		#    ans = ('Hel', '', 'World')
+		#     >>> threeway_split('Hello World','Hel oi World')
+		prefix=longest_common_prefix(old_text,new_text)
+		old_text=old_text[len(prefix):]
+		new_text=new_text[len(prefix):]
+		suffix=longest_common_suffix(old_text,new_text)
+		old_text=old_text[:-len(suffix)]
+		new_text=new_text[:-len(suffix)]
+		middle=new_text
+		return prefix,middle,suffix
+	class DifferentialPython:
+		def __init__(self):
+			self.valid_text=""
+		def update(self,new_text):
+			if is_valid_python_syntax(new_text):
+				self.valid_text=new_text
+				print(fansi(new_text,'green'))
+			else:
+				prefix,middle,suffix=threeway_split(self.valid_text,new_text)
+				if is_valid_python_syntax(prefix+suffix):
+					self.valid_text=prefix+suffix
+					print(fansi(prefix,'green')+fansi(middle,'red','underlined')+fansi(suffix,'green'))
+				else:
+	d=DifferentialPython()
+	SMODIFIER SET d.update(|)
+
+
+
+##############################
+
+
+
+
+
+
+
 
 
 
@@ -81,25 +186,32 @@ KEYBOARD SHORTCUTS:
 	⌥ ↵    run current cell and clear buffer no-matter where you cursor is
 	⇧ ⇥    unindent current line or cycle through completions
 	⌃ ␣    toggle comment on current line and move cursor to next line
-	⌥ ⌃ v  edit current text in vim (equivalent to the \vi microcompletion)
+	⌥ v    edit current text in vim (equivalent to the \vi microcompletion)
 	⌥ ␣    forcibly insert a space character, skipping any microcompletions
 	⌥ ⌫    backspace a whole word
 	⌥ ←    jump back one word
 	⌥ →    jump forward one word
-	⌥ ↓    jump down several lines
-	⌥ ↑    jump up several lines
+	⌥ ↓    jump down 10 lines
+	⌥ ↑    jump up 10 lines
 	⌥ !    run shell command
 	⌥ #    comment out the entire buffer and execute it
-	⌃ t    previous buffer in history regardless of where cursor is (stands for top)
-	⌃ b    next buffer in history regardless of where cursor is (stands for bottom)
+	⌥ n    executes 'NEXT', changing the value of 'ans' (useful in combination with \rpa and \an)
+	⌥ p    executes 'PREV', changing the value of 'ans' (useful in combination with \rpa and \an)
+	⌥ r    similar to \rpa, except it clears the buffer
+	⌥ q    equivalent to \da: clears the current buffer
+	⌥ (    insert ( at current position and ) at the end of the line without moving the cursor to the end of the line
+	⌥ [    insert [ at current position and ] at the end of the line without moving the cursor to the end of the line
+	⌥ )    insert ) at the end of the line without moving the cursor to the end of the line
+	⌥ ]    insert ] at the end of the line without moving the cursor to the end of the line
+	⌃ t    loads previous buffer in history regardless of where cursor is (stands for top). Faster than spamming the up arrow key
+	⌃ b    loads next buffer in history regardless of where cursor is (stands for bottom). Faster than spamming the down arrow key
 	LESS IMPORTANT AND MAYBE REMOVED IN FUTURE:
 		//While these keyboard shortcuts are kind of silly, I do find myself using them quite often
 		⌃ h    inserts 'HISTORY'
 		⌃ p    inserts 'PREV' (then 'NEXT', if pressed again)
 		⌥ m    inserts 'MORE' (then 'MMORE', if pressed again)
 		⌥ s    inserts 'self'
-		⌥ )    insert ) at the end of the line without moving the cursor to the end of the line
-		⌥ ]    insert ] at the end of the line without moving the cursor to the end of the line
+		⌥ ⌃ v  edit current text in vim (equivalent to the \vi microcompletion)
 
 MICROCOMPLETIONS:
 	//This section covers only the microcompletions that take place in the context of a single line
@@ -114,6 +226,8 @@ MICROCOMPLETIONS:
 			\go   goto            (used like ‹`10\go›. Bring the cursor to that line number.)
 			\dtl  delete to line  (used like ‹`5\dtl›. Like d5G in vim.)
 			\ca   cancel          (cancels any command you would have written and deletes the arguments. More civilized than spamming undo or backspace.)
+			\?c   source code     (will enter the source code of any expression given in the first argument. Can also be used with no `s, where the entire buffer is the expression)
+			\/c   source code     (is an alias for ?c)
 		WITHOUT ARGUMENTS:
 			\lo   load file               (loads a file just like `file.py\lo does, except it opens a dialog to let you choose the file so you dont have to type it manually)
 			\db   debugger                (toggles the existence of 'debug()' on the top of your prompt without moving your cursor)
@@ -125,6 +239,7 @@ MICROCOMPLETIONS:
 			\co   copy                    (copies the current buffer to your clipboard)
 			\pa   paste                   (equivalent to control+v)
 			\spa  string paste            (pastes current clipboard as a python string literal)
+			\3pa  commented paste         (pastes current clipboard, but commented out. Can also use this command with arguments: `    \3pa   makes comments with four spaces of indentation, for example.)
 			\wco  web copy                (applies WCOPY to the current buffer's string. Use this to copy text between computers)
 			\wpa  web paste               (inserts the output of WPASTE at your cursor position. Use this to paste text from another computer.)
 			\wspa web string paste        (inserts the output of repr(WPASTE) at your cursor position. Use this to paste text from another computer.)
@@ -134,7 +249,12 @@ MICROCOMPLETIONS:
 			\tco  tmux copy               (applies TCOPY to the current buffer's string. Use this to copy text to a tmux session)
 			\tpa  tmux paste              (inserts the output of TPASTE at your cursor position. Use this to paste text from a tmux session. This is useful because often tmux will press the enter key when pasting a multiline string.)
 			\tspa tmux string paste       (inserts the output of repr(TPASTE) at your cursor position. Use this to paste text from a tmux session. This is useful because often tmux will press the enter key when pasting a multiline string.)
-			\an   ans                     (inserts  str(ans) at current cursor position)
+			\t3pa tmux commented paste    (inserts the output of TPASTE, but commented out. See \3pa for more information.)
+			\vco  vim copy                (applies VCOPY to the current buffer's string. Use this to copy text to vim's default register)
+			\vpa  vim paste               (inserts the output of VPASTE at your cursor position. Use this to paste text vim's default register)
+			\vspa vim string paste        (inserts the output of repr(VPASTE) at your cursor position. Use this to paste text vim's default register as an escaped string)
+			\an   ans                     (inserts str(ans) at current cursor position)
+			\3an  commented ans           (inserts str(ans), but commented out. See \3pa for more information.)
 			\sa   string ans              (inserts repr(ans) at current cursor position)
 			\da   delete all              (deletes all text in the current buffer)
 			\sw   strip whitespace        (strips all trailing whitespace in the buffer)
@@ -143,6 +263,8 @@ MICROCOMPLETIONS:
 			\rl   reverse lines           (reversed the order of all lines of text in the buffer)
 			\sl   sort lines              (sorts all lines in the buffer in alphabetical order)
 			\ya   yapf autoformatter      (autoformats your current code buffer using google's yapf library)
+			\bla  black autoformatter     (autoformats your current code buffer using the 'black' library: pypi.org/project/black )
+			\sim  sort imports            (sorts and organizes all imports in the current buffer, using the 'isort' library)
 			\fn   function name           (writes the name of the current function)
 			\gg   go to top               (like gg in vim. bring cursor to the top of the buffer)
 			\GG   go to bottom            (like G in vim. bring cursor to the bottom of the buffer)
@@ -158,7 +280,13 @@ MICROCOMPLETIONS:
 			\lss  LS SEL                  (lets you select a file or folder and inserts the absolute path as a string)
 			\lsr  (Relative) LS SEL       (lets you select a file or folder and inserts the relative path as a string)
 			\rpr  repr                    (is equivalent to `repr\py  It will turn your buffer into a string literal)
+			\fi   from import swap        (when you're on a line that goes like "from x import y", turns it into "import x". Vice versa.)
+			\en   enumeratify             (Use when in a for-loop block header. Turns whatever it is into enumerated form. For example, ‹for x in y:› to ‹for |,x in enumerate(y):)
 			\rpa  repr to ans             (sets ans to the current buffer's text)
+			\lg   load gist               (treats the current buffer as a Github Gist URL, and tries to load it into the buffer. If it's a git.io shortened url, you only need to specify the title (for example, if using git.io/A3FC you can just use A3FC))
+			\sg   save gist               (saves the current buffer as a Github Gist and sets 'ans' to the url of the gist)
+			\dtb  delete to bottom        (deletes all lines below the cursor. Useful for getting lines after something like CHIST then \pa)
+			\dtt  delete to top           (deletes all lines above the cursor. Useful for getting lines after something like CHIST then \pa)
 
 	FOR LOOPS:
 		COMPREHENSION:
@@ -185,7 +313,7 @@ MICROCOMPLETIONS:
 			in or and if/else,...
 		AUTO SPACES:
 			in, is, as
-		CONTROL FLOW:
+		CONTROL FLOW:shorturl.at/oIY16
 			ON ENTER:
 				try,else,elif,def,...
 				try: except as: pass
@@ -206,7 +334,7 @@ MICROCOMPLETIONS:
 		‹c Name object int›  –––>  ‹class Name(object,int¦):›
 	QUICK KEYWORDS:
 		‹c↵›  –––>  ‹class _:
-		                 ¦›    //Only if we're not directly in a for/while loop
+						 ¦›    //Only if we're not directly in a for/while loop
 
 		‹c ›  –––>  ‹class ¦:›
 
@@ -238,7 +366,7 @@ MICROCOMPLETIONS:
 		‹p↵›  –––>  ‹pass↵›
 
 		‹i↵›  –––>  ‹if True:
-		                 ¦›
+						 ¦›
 
 		‹i ›  –––>  ‹if ¦:› //Only if we're not on the first line (we're more likely to import as a command than start our buffer with an 'if' branch)
 
@@ -251,7 +379,7 @@ MICROCOMPLETIONS:
 		‹l › –––>  ‹lambda ¦:›
 
 		‹w↵›  –––>  ‹while True:
-		                 ¦›
+						 ¦›
 
 		‹w ›  –––>  ‹while ¦:›
 
@@ -260,7 +388,7 @@ MICROCOMPLETIONS:
 		‹wi x › –––>  ‹with x as ¦:›
 
 		‹f↵›  –––>  ‹for _ in ans:
-		                 ¦›
+						 ¦›
 
 		‹f ›  –––>  ‹for ¦ in :› //Only if we're not on the first line
 
@@ -271,45 +399,45 @@ MICROCOMPLETIONS:
 		‹fr › –––>  ‹from ¦ import›  //This works regardless of the line we're on
 
 		‹d↵›  –––>  ‹def ans():
-		                 ¦›              //Only if we're not inside a class
+						 ¦›              //Only if we're not inside a class
 
 		‹d ›       –––> ‹def ¦():›       //Only if we're not inside a class
 
 		‹d↵›  –––>  ‹def __init__(self):
-		                 ¦›              //Only if we are inside a class
+						 ¦›              //Only if we are inside a class
 
 		‹d ›       –––> ‹def ¦(self):›   //Only if we are inside a class
 
 		‹t ›  –––>  ‹try:¦›
 		‹t↵›  –––>  ‹try:
-		                 ¦›
+						 ¦›
 
 
 		‹ex ›  –––>  ‹except ¦:›
 
 		On ‹↵›:  ‹try:       –––\    ‹try:
-		              pass       \        pass
-		              e¦›        /    except:
-		                     –––/         ¦›
+					  pass       \        pass
+					  e¦›        /    except:
+							 –––/         ¦›
 
 		On ‹ ›:  ‹try:               ‹if x:
-		              pass     –––>       pass
-		          ex¦›                except ¦:›
+					  pass     –––>       pass
+				  ex¦›                except ¦:›
 
 		//Each transition arrow is another successive press of the spacebar
 		On ‹ ›:  ‹try:              ‹try:               ‹try:               ‹try:
-		              pass    –––>       pass    –––>       pass    –––>       pass
-		          ex¦                except:¦›           except:¦›           except:pass¦›
+					  pass    –––>       pass    –––>       pass    –––>       pass
+				  ex¦                except:¦›           except:¦›           except:pass¦›
 
 
 		On ‹↵›:  ‹if x:    –––\    ‹if x:
-		              y        \        y
-		              e¦›      /    else:     //enter to else
-		                   –––/         ¦›
+					  y        \        y
+					  e¦›      /    else:     //enter to else
+						   –––/         ¦›
 
 		On ‹ ›:  ‹if x:            ‹if x:            ‹if x:
-		              y     –––>        y     –––>        y    //space to elif to else
-		              e¦›           elif ¦:›          else:¦›
+					  y     –––>        y     –––>        y    //space to elif to else
+					  e¦›           elif ¦:›          else:¦›
 	FUNCTION DEFINITIONS:
 
 		‹d ›       –––> ‹def ¦():›     //Only if we're not inside a class
@@ -501,7 +629,7 @@ MICROCOMPLETIONS:
 				‹def -func-name-(arg-1,-arg-2,-,--):›  –––>  ‹def _func_name_(arg_1,_arg_2,_,__):›
 				‹lambda -x,y-:None›                    –––>  ‹lambda _x,y_:None› //TODO
 				On ‹-›: ‹def f():     –––\   ‹def f():
-				             var¦›    –––/        var_¦›   //Why would you start a line in a function with an expression like x-y? That seems fairly useless, almost all the time. Instead, turn - into _ without having to double-tap
+							 var¦›    –––/        var_¦›   //Why would you start a line in a function with an expression like x-y? That seems fairly useless, almost all the time. Instead, turn - into _ without having to double-tap
 				EDGE CASES:
 					‹x--›  –/–> ‹x__›  //This is because of a different microcompletion: the -- operator, which makes ‹x--› –––> ‹x-=1¦›
 
@@ -686,40 +814,58 @@ MICROCOMPLETIONS:
 
 PSEUDO TERMINAL COMMANDS:
 	SUMMARY:
-        <Input Modifier>     <Namespace History>    <Profiler>         <Documentation>       
-        MODIFIER ON          UNDO                   PROF               HELP                  
-        MODIFIER OFF         UNDO ON                PROF ON            HHELP                 
-        MODIFIER SET         UNDO OFF               PROF OFF                                 
-        SMODIFIER SET        UNDO CLEAR             PROF DEEP          <Module Reloading>    
-                             UNDO ALL                                  RELOAD ON             
-        <Stack Traces>                              <Toggle Colors>    RELOAD OFF            
-        MORE                 <Prompt Toolkit>       FANSI ON                                 
-        MMORE                PT ON                  FANSI OFF          <Unimportant>         
-        DMORE                PT OFF                                    IPYTHON               
-        AMORE                                       <Others>           IPYTHON ON            
-        GMORE                <Saving Settings>      RETURN             IPYTHON OFF           
-        VIMORE               PT SAVE                SUSPEND            RETURN NOW            
-                             PT RESET               SHELL              RYAN RPRC             
-        <Command History>    SET STYLE              LEVEL              GC OFF                
-        HISTORY                                     DITTO              GC ON                 
-        AHISTORY             <Shell Commands>       EDIT                                     
-        GHISTORY             !                      VARS                                     
-        CHISTORY             !!                     RANT                                     
-        DHISTORY                                    FORK                                     
-                             <Inspection>           RPRC                                     
-        <Clipboard>          ?/                     ARG                                      
-        COPY                 ?.                                                              
-        PASTE                ?                      <File System>                            
-        SPASTE               ??                     CD                                       
-                             ???                    LS                                       
-        <'ans' History>      ????                   PWD                                      
-        NEXT                 ?????                  CAT                                      
-        PREV                                        RUN                                      
-        PREV ON              <Simple Timer>         CDP                                      
-        PREV OFF             TICTOC                 CPWD                                     
-        PREV CLEAR           TICTOC ON              NCAT                                     
-        PREV ALL             TICTOC OFF             CCAT                                     
-                                                    OPEN          
+		ALL COMMANDS:
+			<Input Modifier>        <Namespace History>    <Startup Files>          <Unimportant>       
+			MODIFIER ON             UNDO                   RPRC                     NUM COM             
+			MODIFIER OFF            UNDO ON                VIMRC                    IHISTORY (IHIST)    
+			MODIFIER SET            UNDO OFF               TMUXRC                   PROF DEEP           
+			SMODIFIER SET           UNDO CLEAR             XONSHRC                  CDH CLEAN           
+			                        UNDO ALL               RYAN RPRC                                    
+			<Stack Traces>                                 RYAN VIMRC               <File System>       
+			MORE                    <Prompt Toolkit>       RYAN TMUXRC              RM                  
+			MMORE                   PT ON                  RYAN XONSHRC             LS                  
+			DMORE                   PT OFF                                          FD                  
+			AMORE                   PT                     <Inspection>             CD                  
+			GMORE                                          ?                        CDP                 
+			HMORE                   PT SAVE                ??                       CDA                 
+			VIMORE                  PT RESET               ???                      CDB                 
+			PIPMORE                 <RP Settings>          ?.                       CDU                 
+			IMPMORE                 SET STYLE              ?v                       CDH                 
+			PREVMORE                                       ?s                       CDZ                 
+			NEXTMORE                <Shell Commands>       ?t                       CDQ                 
+			                        !                      ?h (?/)                  CAT                 
+			<Command History>       !!                     ?e                       NCAT                
+			HISTORY    (HIST)                              ?p                       CCAT                
+			GHISTORY   (GHIST)      <Simple Timer>         ?c                       ACAT                
+			AHISTORY   (AHIST)      TICTOC                                          CATA                
+			CHISTORY   (CHIST)      TICTOC ON              <Others>                 NCATA               
+			DHISTORY   (DHIST)      TICTOC OFF             RETURN  (RET)            CCATA               
+			VHISTORY   (VHIST)                             SUSPEND (SUS)            ACATA               
+			ALLHISTORY (ALLHIST)    <Profiler>             GPU                      RUN                 
+			                        PROF                   TOP                      RUNA                
+			<Clipboards>            PROF ON                TAB                      PWD                 
+			COPY                    PROF OFF               TABA                     CPWD                
+			PASTE                                          MONITOR                  APWD                
+			EPASTE                  <Toggle Colors>        UPDATE                   TAKE                
+			WCOPY                   FANSI ON               ANS PRINT ON   (APON)    OPEN                
+			WPASTE                  FANSI OFF              ANS PRINT OFF  (APOF)    OPENH               
+			TCOPY                                          ANS PRINT FAST (APFA)    OPENA               
+			TPASTE                  <Module Reloading>     SHELL (SH)               DISK                
+			LCOPY                   RELOAD ON              LEVEL                    TREE                
+			LPASTE                  RELOAD OFF             DITTO                    TREE ALL            
+			VCOPY                                          EDIT                     TREE DIR            
+			VPASTE                  <Documentation>        VARS                     TREE ALL DIR        
+			FCOPY                   HELP                   RANT                     FD SEL (FDS)        
+			FPASTE                  HHELP                  FORK                     LS SEL (LSS)        
+			MLPASTE                 SHORTCUTS              WANS                     LS REL (LSR)        
+			                                               ARG                      LS FZF (LSZ)        
+			<'ans' History>                                VIM                      LS QUE (LSQ)        
+			NEXT                                           VIMH                     RANGER (RNG)        
+			PREV                                           VIMA                                         
+			PREV ON                                        AVIMA                                        
+			PREV OFF                                       GC OFF                                       
+			PREV CLEAR                                     GC ON                                        
+			PREV ALL                                       GC       
 
 	<Input Modifier>
 		// In pseudo-terminal, modifiers are preprocessors for your code.
@@ -809,6 +955,20 @@ PSEUDO TERMINAL COMMANDS:
 		PROF OFF: Turns the profiler off.
 		PROF: Toggles PROF ON with PROF OFF. Quick and dirty.
 
+	?c  gets source code
+	?p  prints either with pretty_print, or prints a python code string with syntax highlighting
+	TOP vs MON: MON is simple, and always colored properly. TOP is not always colored properly depending on the terminal, but when it is it's very pretty. 
+	?.  ans?.query   or   ans?.2  for fzf seach 2 layers deep+selection
+	CDA will cd into a module's directory if ans is a module; into the source file's folder for an object (if possible, in same way that ?v works), or into the parent folder if ans is a path to a file
+	CAT will work on URL's too, kind of like curl.
+	CAT will syntax-highlight any python files
+	ACAT also works on images, videos, and sounds - as well as any text file or url
+	AVIMA will highlight a string using python syntax highlighting, if the string is valid python syntax
+	OPENA will open a folder, file OR url in your web browser!
+	LEVEL will print out the current pseudo_terminal level, as well as the current python version and computer name. Useful when running RP on multiple computers and you need to kno who's who.
+
+
+
 
 
 
