@@ -128,6 +128,9 @@ class PythonCompleter(Completer):
             char_before_cursor.isalnum() or char_before_cursor in '_.')
 
     def _get_completions(self, document, complete_event,force=False):# When force is true it acts like the original ptpython autocomplete
+      import warnings
+      with warnings.catch_warnings():
+        warnings.simplefilter('ignore')
         force=True
         global old_origin,candidates
         origin=document.get_word_before_cursor()
@@ -311,6 +314,9 @@ class PythonCompleter(Completer):
 
         old_origin=origin
     def get_completions(self, document, complete_event,force=False):
+      import warnings
+      with warnings.catch_warnings():
+        warnings.simplefilter('ignore')
 
         global completion_cache_pre_origin_doc
         from rp import tic,toc,ptoctic,ptoc 
