@@ -253,7 +253,8 @@ def signature_toolbar(python_input):
                         append((Signature,desc_string))
                     append((Signature.Operator, ', '))
                 except:
-                    print("(Just caught a bug that would have crashed the prompt-toolkit gui. Why did this happen??)")
+                    #NOTE: This used to be printed...but seeing as this bug never seemed to cause any problems and only spammed the terminal, I'm getting rid of it. I bet you'll never notice...
+                    # print("(Just caught a bug that would have crashed the prompt-toolkit gui. Why did this happen??)")
                     pass#I don't know why, but this broke pseudo terminal. I don't know what this is but I don't really care...it's not worth crashing over...
 
             if sig.params:
