@@ -57,6 +57,14 @@ def set_debug_height(height):
     ric.debug_height=height
     return height
 
+def set_history_line_limit(number_of_lines):
+    from rp.r import _globa_pyin
+    _globa_pyin[0].history_number_of_lines=number_of_lines
+    return number_of_lines
+def get_history_line_limit():
+    from rp.r import _globa_pyin
+    return _globa_pyin[0].history_number_of_lines
+
 class OptionCategory(object):
     def __init__(self,title,options):
         assert isinstance(title,six.text_type)
@@ -499,6 +507,60 @@ class PythonInput(object):
                 simple_option(title='Wrap lines',
                               description='Wrap lines instead of scrolling horizontally.',
                               field_name='wrap_lines'),
+                                simple_option(title='History Highlighting',
+                              description='When using F3 (aka History Broswer) to select entries from history,  '
+                                          'should we use syntax highlighting? Its pretty, but slow.',
+                              field_name='history_syntax_highlighting'),
+                Option(title='History Browser Limit',
+                        is_visible=lambda:getattr(self,'show_all_options',True),
+
+                       description='How many lines should we display when using F3 history? Less is faster',
+                       get_current_value=lambda:str(get_history_line_limit()).rjust(12),
+                       get_values=lambda:{'asdpoa':lambda:None,
+                            '           1':lambda:set_history_line_limit(1          ),
+                            '           5':lambda:set_history_line_limit(5          ),
+                            '          10':lambda:set_history_line_limit(10          ),
+                            '          20':lambda:set_history_line_limit(20          ),
+                            '          30':lambda:set_history_line_limit(30          ),
+                            '          40':lambda:set_history_line_limit(40          ),
+                            '          50':lambda:set_history_line_limit(50          ),
+                            '         100':lambda:set_history_line_limit(100         ),
+                            '         150':lambda:set_history_line_limit(150         ),
+                            '         200':lambda:set_history_line_limit(200         ),
+                            '         250':lambda:set_history_line_limit(250         ),
+                            '         300':lambda:set_history_line_limit(300         ),
+                            '         400':lambda:set_history_line_limit(400         ),
+                            '         500':lambda:set_history_line_limit(500         ),
+                            '         600':lambda:set_history_line_limit(600         ),
+                            '         700':lambda:set_history_line_limit(700         ),
+                            '         800':lambda:set_history_line_limit(800         ),
+                            '         900':lambda:set_history_line_limit(900         ),
+                            '        1000':lambda:set_history_line_limit(1000        ),
+                            '        1500':lambda:set_history_line_limit(1500        ),
+                            '        2000':lambda:set_history_line_limit(2000        ),
+                            '        2500':lambda:set_history_line_limit(2500        ),
+                            '        3000':lambda:set_history_line_limit(3000        ),
+                            '        3500':lambda:set_history_line_limit(3500        ),
+                            '        4000':lambda:set_history_line_limit(4000        ),
+                            '        4500':lambda:set_history_line_limit(4500        ),
+                            '        5000':lambda:set_history_line_limit(5000        ),
+                            '        6000':lambda:set_history_line_limit(6000        ),
+                            '        7000':lambda:set_history_line_limit(7000        ),
+                            '        8000':lambda:set_history_line_limit(8000        ),
+                            '        9000':lambda:set_history_line_limit(9000        ),
+                            '       10000':lambda:set_history_line_limit(10000       ),
+                            '       15000':lambda:set_history_line_limit(15000       ),
+                            '       20000':lambda:set_history_line_limit(20000       ),
+                            '       25000':lambda:set_history_line_limit(25000       ),
+                            '       30000':lambda:set_history_line_limit(30000       ),
+                            '       40000':lambda:set_history_line_limit(40000       ),
+                            '       50000':lambda:set_history_line_limit(50000       ),
+                            '      100000':lambda:set_history_line_limit(100000      ),
+                            '     1000000':lambda:set_history_line_limit(1000000     ),
+                            '    10000000':lambda:set_history_line_limit(10000000    ),
+                            '999999999999':lambda:set_history_line_limit(999999999999),
+                       }),
+
                 Option(title='Show Battery Life',
 
                         is_visible=lambda:getattr(self,'show_all_options',True),
