@@ -93,9 +93,11 @@ nmap <F8>     :source ~/.Session.vim<CR>:Obsession ~/.Session.vim<CR>
 nmap <F8><F8> :Obsession ~/.Session.vim<CR>
 
 nmap td :tab split<cr>
+nmap tn :tabnew<cr>
 nmap ts :tab split<cr>
 nmap tc :tab close<cr>
 nmap tq :quit<cr>
+nmap \co gg"+yG<c-o>zz
 nmap <esc>q tq
 nnoremap <esc>w <c-w>
 
